"use strict";
this.name           = "SaveInFlight";
this.author         = "Dybal";
this.copyright      = "2022 Dybal";
this.license        = "CC-BY-NC-SA 4.0";
this.description    = "Allows saving the game in flight";
this.version        = "1.0.1";

this.$debug = false;
this.$logging = true;
this.$sifnabled = false;
this.$position; // player ship saved position
this.$orientation; // player ship saver orientation
this.$fuel_leak; // playert ship fuel leak rate
this.$unmarshalled_commodities; // list of manifest.list items (the whole object) unmarshalled
this.$marshallable_commodities; // list of ids for the commodities that can be marshalled
this.$sif_station = null; // Station at the End of Universe :-)

//
// Event Handlers
//

//------------------------------------------------------------------------------------------//
this.startUpComplete = function _SaveInFlight_startUpComplete() {
    this.ship = player.ship;
    if (this.ship.equipmentStatus("EQ_SAVE_IN_FLIGHT") === "EQUIPMENT_OK")
        this.$sifEnabled = true;
    else
        this.$sifEnabled = false;
    if (missionVariables.SaveInFlight_position) {
        var pos = JSON.parse(missionVariables.SaveInFlight_position);
        var or = JSON.parse(missionVariables.SaveInFlight_orientation);
        this.$position = new Vector3D(pos)
        this.$orientation = new Quaternion(or);
        if (missionVariables.SaveInFlight_fuel_leak_rate) {
            this.$fuel_leak = parseFloat(missionVariables.SaveInFlight_fuel_leak_rate);
            delete missionVariables.SaveInFlight_fuel_leak_rate;
        }
        if (missionVariables.SaveInFlight_unmarshalled_commodities) {
            this.$unmarshalled_commodities = JSON.parse(missionVariables.SaveInFlight_unmarshalled_commodities);
            delete missionVariables.SaveInFlight_unmarshalled_commodities;
        }
        delete missionVariables.SaveInFlight_position;
        delete missionVariables.SaveInFlight_orientation;
    }
    if (worldScripts.BGS) {
        //this.$bg = getScreenBackgroundForKey("shiplibrary");
        this.$bg = {name:"bgs_status_void.png"};
    } else { 
        this.$bg = {name:"splash@2x.png"};
    }
    this.$marshallable_commodities = this.$get_marshallable_commodities();
    log(this.name, "Marshallable commodities: "+this.$marshallable_commodities);
}

//------------------------------------------------------------------------------------------//
this.systemWillPopulate = function _SaveInFlight_systemWillPopulate() {
    log(this.name, "SystemWillPopulate");
    system.setPopulator("End of Universe Station", {
        callback: function(pos) {
            var ws = worldScripts.SaveInFlight;
            var s = system.addShips("sif_station", 1, pos, 0);
            if (s && s.length == 1) {
                ws.$sif_station = s[0];
                ws.$sif_station.displayName = "Station at the End of Universe";
                ws.$sif_station.breakPattern = false;
                log(ws.name, "Created station "+ws.$sif_station);
            } else {
                log(ws.name, "Could not create the Station at the End of Universe");
            }
        }.bind(this),
        location: "COORDINATES",
        coordinates: Vector3D(0, 0, -1E4).fromCoordinateSystem("spu"),
        deterministic: true
    });
}

//------------------------------------------------------------------------------------------//
this.equipmentAdded = function _SaveInFlight_equipmentAdded(equipmentKey) {
    if (equipmentKey === "EQ_SAVE_IN_FLIGHT") {
        if (this.$debug) log(this.name, "Save in Flight added")
        this.$sifEnabled = true;
    }
}

//------------------------------------------------------------------------------------------//
this.equipmentRemoved = function _SaveInFlight_equipmentRemoved(equipmentKey) {
    if (equipmentKey === "EQ_SAVE_IN_FLIGHT") {
        if (this.$debug) log(this.name, "Save in Flight removed")
        this.$sifEnabled = false;
    }
}

//------------------------------------------------------------------------------------------//
this.playerWillSaveGame = function _SaveInFlight_playerWillSaveGame() {
    var pos = this.$position;
    var or = this.$orientation;
    if (this.$position) {
        missionVariables.SaveInFlight_position = JSON.stringify([pos.x, pos.y, pos.z]);
        missionVariables.SaveInFlight_orientation = JSON.stringify([or.w, or.x, or.y, or.z]);
        if (this.$fuel_leak != null) missionVariables.SaveInFlight_fuel_leak_rate = this.$fuel_leak;
        if (this.$unmarshalled_commodities) missionVariables.SaveInFlight_unmarshalled_commodities = JSON.stringify(this.$unmarshalled_commodities);
    }
}

//------------------------------------------------------------------------------------------//
this.shipLaunchedFromStation = function _SaveInFlight_shipLaunchedFromStation(station)  {
    var p = player.ship;
    if (station == this.$sif_station) {
        log(this.name, "Placing player ship at "+this.$position+" with orientation "+this.$orientation+", speed:" +p.speed);
        var max_speed = p.maxSpeed;
        p.maxSpeed = 0;
        p.position = this.$position;
        p.orientation = this.$orientation;
        p.maxSpeed = max_speed;
        if (this.$fuel_leak != null) p.fuelLeakRate = this.$fuel_leak;
        // unmarshall the scooped marshallable commodities (those in g and kg)
        this.$restore_unmarshalled_commodities();
        this.$position = this.$orientation = this.$fuel_leak = this.$unmarshalled_commodities = null;
    }
}

//------------------------------------------------------------------------------------------//
this.guiScreenChanged = function _SaveInFlight_guiScreenChanged(to, from) {
    var pship = player.ship;
    if (!pship.docked || !pship.dockedStation.hasRole("sif_station")) return;
    if (guiScreen === "GUI_SCREEN_EQUIP_SHIP") {
        mission.runScreen({
            title: "Station at the End of Universe",
            message: "This station has no Shipyard",
            background: this.$bg 
        });
    } else if (guiScreen === "GUI_SCREEN_MARKET") {
        mission.runScreen({
            title: "Station at the End of Universe",
            message: "This station has no Market",
            background: this.$bg 
        });
    } else if (guiScreen === "GUI_SCREEN_INTERFACES") {
        mission.runScreen({
            title: "Station at the End of Universe",
            message: "This station has no Interfaces",
            background: this.$bg 
        });
    }     
}

//------------------------------------------------------------------------------------------//
this.activated = function _SaveInFlight_activated() {
    var ws = worldScripts.SaveInFlight;
    var p = player.ship;
    if (ws.$debug) log(ws.name, "Trying to activate Save In Flight: "+ws.$sifEnabled);
    if (ws.$sifEnabled) {
        if (p.status != "STATUS_IN_FLIGHT") {
            player.consoleMessage("Ship must be in flight for saving", 30);
            return;
        }
        var in_range = p.checkScanner();
        var n=0;
        for (var i=0;i<in_range.length; i++) {
            if (in_range[i].dataKey != "telescopemarker") n++;
        }
        if (n > 0) {
            if (ws.$debug) log(ws.name, "In range: "+in_range);
            player.consoleMessage("There must be nothing in scanner range for saving", 30);
            return;
        }
        if (p.speed > 0.00001) {
            player.consoleMessage("Ship must be stopped for saving", 30);
            return;
        }
        ws.$position = p.position;
        ws.$orientation = p.orientation;
        ws.$fuel_leak = p.fuelLeakRate;
        // store data on scooped marshallable commodities (those in g and kg)
        this.$store_unmarshalled_commodities();       
        if (ws.$debug) log(ws.name, "SaveInFlight activated, ship position: "+ws.$position+" and orientation: "+ws.$orientation);
        if (!ws.$sif_station.dockPlayer()) {
            log(ws.name, "Could not \"dock\" player at the Station at the End of Universe");
        }
    }
}


//------------------------------------------------------------------------------------------//
//  Internal Functions
//------------------------------------------------------------------------------------------//
this.$store_unmarshalled_commodities = function _SaveInFlight_store_unmarshalled_commodities() {
    var ws = worldScripts.SaveInFlight;
    var m = ws.$marshallable_commodities;
    if (m == null) return;
    var manifest_list = manifest.list;
    var unmarshalled = [];
    var c,i;
    i = manifest_list.length;
    while (i--) {
        c = manifest_list[i];
        if (ws.$debug) log(ws.name, "looking at "+c.commodity+", "+c.containers+"TCs");
        if (c.containers > 0 && m.indexOf(c.commodity) !== -1) unmarshalled.push(c);
    }
    if (unmarshalled.length > 0) {
        ws.$unmarshalled_commodities = unmarshalled;
    } else {
        ws.$unmarshalled_commodities = null;
    }
}

//------------------------------------------------------------------------------------------//
this.$restore_unmarshalled_commodities = function _SaveInFlight_store_unmarshalled_commodities() {
    var unmarshalled = this.$unmarshalled_commodities;
    var c, q, c_id, i;
    if (!unmarshalled) return;
    i = unmarshalled.length;
    while (i--) {
        c = unmarshalled[i];
        q = c.containers;
        c_id = c.commodity;
        manifest[c_id] -= q;
        while (q--) manifest[c_id]++;
    }
}

//------------------------------------------------------------------------------------------//
this.$get_marshallable_commodities = function _SaveInFlight_get_unmarshallable_commodities() {
    var m = [];
    var s_mkt = (system.mainStation ? system.mainStation.market : null);
    var i, c;

    for (c in s_mkt) {
        // quantity_units: ton=0, kg=1, g=2
        if (s_mkt[c].quantity_unit > 0) m.push(c);
    }
    return m; // list of ids for the commodities that can be marshalled
}
